(function(){
 'use strict';

 const Ld = sLoader;
 Ld.module('console',function(){

  let Cons = {};

  Cons.show = show;
  Cons.hide = hide;
  Cons.log = log;
  Cons.error = error;
  Cons.toggle = toggle;
  Cons.setBackColor = setBackColor;


  const MIN_FONT_SIZE = 9;
  const TB_HEIGHT = 40;
  const TB_FONT_SIZE = 20;

  let screenEl;
  let frameEl;
  let titleEl;
  let closeEl;
  let bodyEl;
  let status;
  let nLine = 0;
  let maxLine = 1000;
  let level;


  function open(){

   const title = 'Console';

   screenEl = document.createElement('div');

   setStyle(screenEl.style,{
    position:'absolute',
    top:0, bottom:0, left:0, right:0,
    backgroundColor:'rgba(0,0,0,0.4)',
    zIndex:0,
   });
   document.body.appendChild(screenEl);

   frameEl = document.createElement('div');
   setStyle(frameEl.style,{
    position:'absolute',
   });
   screenEl.appendChild(frameEl);

   titleEl = document.createElement('div');
   setStyle(titleEl.style,{
    position:'absolute',
    top:0,left:0,right:TB_HEIGHT+'px',height:TB_HEIGHT+'px',
    fontSize:'1.4em',
    lineHeight:TB_FONT_SIZE+'px',
    boxSizing:'border-box',
    overflow:'hidden',
    padding:`${(TB_HEIGHT-TB_FONT_SIZE)/2}px ${TB_FONT_SIZE/2}px`,
   });
   frameEl.appendChild(titleEl);
   titleEl.innerHTML = title;

   closeEl = document.createElement('div');
   setStyle(closeEl.style,{
     position:'absolute',
     top:0, right:0, width:TB_HEIGHT+'px', height:TB_HEIGHT+'px',
     textAlign:'center',
     lineHeight:TB_HEIGHT+'px',
   });
   closeEl.addEventListener('click',function(ev){
    ev.stopPropagation();
    ev.preventDefault();
    hide();
   });
   frameEl.appendChild(closeEl);
   closeEl.innerHTML = '\u2715';

   bodyEl = document.createElement('div');
   bodyEl.className = 'scrollbox';
   setStyle(bodyEl.style,{
    position:'absolute',
    top:TB_HEIGHT+'px', bottom:0, left:0,right:0,
    padding:'0.25em 0.5em',
    overflow:'scroll',
    WebkitOverflowScrolling:'touch',
   });
   frameEl.appendChild(bodyEl);

   window.addEventListener('resize',function(ev){
    onresize();
   });

   setBackColor();
   onresize();

  }


  const vendorPrefixNeeded = {
   transformOrigin: 1,
   transform      : 1,
   filter         : 1,
  };

  function setStyle(dst,src){
   if ( !src ) { return; }
   for( let property in src ) {
    if ( vendorPrefixNeeded[property] ) {
     let pascal = property.replace(/^([a-z])/,function(match){
      return match.toUpperCase();
     });
     dst['Ms'+pascal] = src[property];
     dst['Moz'+pascal] = src[property];
     dst['Webkit'+pascal] = src[property];
    }
    dst[property] = src[property];
   }
  }


  function setBackColor(backColor){
   let opacity = 0.8;
   let bc = backColor||'255,255,255';
   let c = bc.split(/,/).map(parseFloat);
   // tekitou
   let mono = 0.299*c[0] + 0.587*c[1] + 0.114*c[2];
   let fc = mono >= 128? '0,0,0': '255,255,255';
   let color = `rgba(${fc},0.8)`;
   let bgcolor = `rgba(${bc},${opacity})`;
   setStyle(titleEl.style,{
    backgroundColor:color, color:bgcolor,
   });
   setStyle(closeEl.style,{
    backgroundColor:color, color:bgcolor, fill:bgcolor,
   });
   setStyle(bodyEl.style,{
     backgroundColor:bgcolor, color:color,
   });
  };


  function onresize(ev){
   let ww = window.innerWidth;
   let wh = window.innerHeight;
   const maxWidth = 720;
   const maxHeight = 1080;
   let boxw = Math.min(ww,maxWidth);
   let boxh = Math.min(wh,maxHeight);
   let x = (ww-boxw)/2;
   let y = (wh-boxh)/2;
   setStyle(frameEl.style,{
    left:x+'px', top:y +'px',
    width:boxw+'px', height:boxh+'px',
   });

   let fs = boxw/56;
   if ( fs < MIN_FONT_SIZE ) { fs = MIN_FONT_SIZE; }
   setStyle(screenEl.style,{
    fontSize:fs+'px',
    lineHeight:(fs*1.6)+'px',
   });
  }

 // あったか？
  function htmlEscape(str) {
   if ( str === undefined ) {
    str = '(undefined)';
   }
   else if ( str.toString ) {
    str = str.toString();
   }
   return str.replace(/&/g, "&amp;").
    replace(/"/g, "&quot;").
    replace(/</g, "&lt;").
    replace(/>/g, "&gt;");
  }


  function _out(/*arguments*/){
   let args = [];
   for( let i = 0; i < arguments.length; i++ ) {
    args[i] = arguments[i];
   }
   let message = args.join(' ');
   while( nLine >= maxLine ) {
    bodyEl.removeChild(bodyEl.firstChild);
    nLine--;
   }
   let el = document.createElement('div');
   let htmltext = htmlEscape(message).replace(/\n/,'<br>');
   el.innerHTML = htmltext;
   if ( level == 'error' ) {
    el.style.color = 'red';
   }
   bodyEl.appendChild(el);
   nLine++;
   bodyEl.scrollTop = bodyEl.scrollHeight;
   if ( console ) {
    console.log(message);
   }
  }


  function log(/*arguments*/){
   level = 'info';
   _out.apply(this,arguments);
  }


  function error(/*arguments*/){
   level = 'error';
   _out.apply(this,arguments);
  }


  function show(){
   screenEl.style.visibility = "visible";
   screenEl.style.zIndex = 10;  // Mobile Safari bug!
   status = true;
  }
  
  function hide(){
   screenEl.style.visibility = "hidden";
   screenEl.style.zIndex = 0;   // Mobile Safari bug!
   status = false;
  }

  function toggle(){
   if ( status ) {
    hide();
   } else {
    show();
   }
  }


  open();
  show();

     
  return Cons;

 });
})();
