/*
*/
(function(){
 'use strict';

 const Ld = sLoader;


 Ld.module('cropLayer',function(){

  const Lp   = Ld.use('layerParts');
  const Crop = Lp.derive();

  const M    = Ld.ref('main');
  const U    = Ld.use('utility');
  const Icon = Ld.use('icon');
  const Cp   = Ld.use('cropParam');
  const Mn   = Ld.use('menu');




  const CROP_MASK_OPACITY       = 0.4;
  const CROP_BORDER_OPACITY     = 0.8;

  const MAX_ASPECT_RATIO = 16;

  const PRESET_ASPECT_RATIO = [
   "original",
   "1:2",
   "9:16",
   "2:3",
   "3:4",
   "1:1",
   "4:3",
   "3:2",
   "16:9",
   "2:1",
  ];

  Crop.sig = 'Cropping';

  Crop.defineMenu = function(){
   Crop._super.defineMenu.call(this);
   let obj = this;
   this.addMenu({

// combobox!
    aspectRatio   :
      Mn.CSelect.create({ priority:90,
       title: 'Aspect Ratio',
       default:'original',
       list: PRESET_ASPECT_RATIO,
      }),

    transform       :
      Mn.CTransform.create({priority:85,
//
//       buttonText: 'Move, Resize,<br>Rotate',
       buttonText: Icon.cropRotate('100%'),
       zoom:{ min:1/2, max:16, default:1 },
       rotation:{ default:0 },
       screenToInternalCoordinate:function(pos){
        let p = U.convertFromScreenCoordinate(M.paneEl,pos);
        let cw = M.scale*Cp.width;
        let ch = M.scale*Cp.height;
        return { x:(p.x-cw/2)/M.scale, y:(p.y-ch/2)/M.scale };
       },
      }),
    orientation:
      Mn.CSubmenu.create({priority:60, text:'Orientation',
       submenuId:'cropOrientation',
      }),
    cropMaskColor: Mn.CSlider.create({priority:5,
       title: 'Crop Mask<br>Brightness',
       min: 0, max: 100, default: 25,
      }),
   });

   Mn.CMenuDef.create({
    id:'cropOrientation',
    items:{
     rotate90      :
       Mn.rotate90({priority:45,
        onclick:function(){
         obj.changeOrientation(0.25); // radianかdegreeに!
        }
       }),
     rotate270     :
       Mn.rotate270({priority:40,
        onclick:function(){
         obj.changeOrientation(-0.25); // radianかdegreeに!
        }
       }),
     flipH     :
       Mn.CIconButton.create({priority:15,
        text: Icon.flipH('100%'),
        onclick:function(){ obj.flipH(); }
       }),
     flipV     :
       Mn.CIconButton.create({priority:10,
        text: Icon.flipV('100%'),
        onclick:function(){ obj.flipV(); }
       }),
    },
   });
  };



  Crop.onresize = function(){
   this.dirty = true;
   M.disp();
  };


// aspect比が変更された場合
  Crop.disp = function(){
   if ( M.imageReady && this.dirty ) {
//
if ( isNaN(M.cRect.width) ) return;
    this.dirty = false;
    let ox = -M.cRect.diagonal;
    let oy = -M.cRect.diagonal;
    let ow = M.cRect.width+2*M.cRect.diagonal;
    let oh = M.cRect.height+2*M.cRect.diagonal;
    let cx = 0;
    let cy = 0;
    let cw = M.cRect.width;
    let ch = M.cRect.height;
    let opacity, color;
    if ( Cp.cropTranslucent ) {
     color = 'black'; opacity = CROP_MASK_OPACITY;
    } else {
     let v = Math.round(Cp.cropMaskColor/100*255);
     if ( v < 0 ) { v = 0; }
     if ( v > 255 ) { v = 255; }
     color = `rgb(${v},${v},${v})`;
     opacity = 1;
    }
    let svg =
    `<svg viewBox="${ox} ${oy} ${ow} ${oh}"
       x="${ox}px" y="${oy}px" width="${ow}" height="${oh}"
       style="position:absolute; top:${oy}px; left:${ox}px;"
     >
     <path d="M ${ox} ${oy} h ${ow} v ${oh} h ${-ow} v ${-oh} Z
       M ${cx} ${cy} v ${ch} h ${cw} v ${-ch} h ${-cw} Z"
      fill="${color}" opacity="${opacity}" />
     <rect x="${cx}" y="${cy}" width="${cw}" height="${ch}"
      stroke="white" stroke-width="1" fill="none" />
     </svg>`;
    this.containerEl.innerHTML = svg;
   }
  };

/*
  Crop.onMenuActive = function(){
console.log('onMenuActive');
//   M.cropMode = true;
   M.adjustScale();
   M.redraw();
  };

  Crop.onMenuInactive = function(){
console.log('onMenuInactive');
//   M.cropMode = false;
   M.adjustScale();
   M.redraw();
  };
*/

  Crop.onOpenImage = function(){
   this.menu.setValues({ aspectRatio:'original' });
  };


  Crop.onChange = function(values){
   Object.assign(Cp,values);
   if ( values.aspectRatio !== undefined ) {
    this.changeAspect();
   }
   if ( values.cropTranslucent !== undefined
     || values.cropMaskColor !== undefined ) {
    this.dirty = true;
   }

   if ( values.transform ) {
// 画像レイヤだけ?を書き直さないと
// redrawImage
    M.redraw();
   }
   else {
//?
    M.disp();
   }
   
//   Crop._super.onChange.call(this,values);
  };


// これでは古いバージョンで不要になった情報も蓄積されていくな。
  Crop.restoreSessionStatus = function(data){
   this.menu.restore(data);
   Object.assign(Cp,this.menu.getValues());
   if ( data ) {
    Cp.diagonal      = 1;
    Cp.width         = data.width/U.dist(data.width,data.height);
    Cp.height        = data.height/U.dist(data.width,data.height);
    Cp.mirror        = !!data.mirror;
    Cp.aspectH       = data.aspectH;
    Cp.aspectV       = data.aspectV;
   }
  };

  Crop.getSessionStatus = function(){
   return Cp;
  };

  Crop.loadSetting = function(data){
   this.menu.restore(data);
   Object.assign(Cp,this.menu.getValues());
  };

  Crop.getSetting = function(){
   return {
    cropTranslucent: Cp.cropTranslucent,
    cropMaskColor  : Cp.cropMaskColor,
   };
  };

  Crop.setTranslucent = function(b){
   Cp.cropTranslucent = b;
   this.menu.setValues({ cropTranslucent: b });
   this.dirty = true;
   M.disp();
  };

/*
  function rotate(rotation,cx,cy){
   let t = 2*Math.PI*rotation;
   let x = Cp.transform.x-cx;
   let y = Cp.transform.y-cy;
   Cp.transform.x = x*Math.cos(t) - y*Math.sin(t) + cx;
   Cp.transform.y = x*Math.sin(t) + y*Math.cos(t) + cy;
   Cp.transform.rotation += rotation;
  }
*/

  Crop.changeAspect = function(){
   let comp = Cp.aspectRatio.split(/:/,2);
   let h = parseInt(comp[0]);
   let v = parseInt(comp[1]);
   if ( isNaN(h) || isNaN(v) || h == 0 || v == 0 
     || h/v > MAX_ASPECT_RATIO || v/h > MAX_ASPECT_RATIO ) {
    //
    h = Mp.imageWidth; v = Mp.imageHeight;
    if ( h === undefined || v === undefined ) {
     h = 1; v = 1;
    }
   }
   let d = U.gcd(h,v);
   h /= d;
   v /= d;
   M.setAspectRatio(h,v);
   M.redrawImmediately();
  };

  Crop.changeOrientation = function(rotation){
   // rotation: 0.25 or -0.25
   let t = 2*Math.PI*rotation;
   let x = Cp.transform.x;
   let y = Cp.transform.y;
   Cp.transform.x = x*Math.cos(t) - y*Math.sin(t);
   Cp.transform.y = x*Math.sin(t) + y*Math.cos(t);
   let r = Cp.transform.rotation+rotation;
   Cp.transform.rotation = r - Math.floor(r);
   M.setAspectRatio(Cp.aspectV,Cp.aspectH);
   M.adjustScale();
//   M.redrawImmediately();
  };

  Crop.flipH = function(){
   Cp.mirror = !Cp.mirror;
   Cp.transform.rotation = 1-Cp.transform.rotation;
   Cp.transform.x = -Cp.transform.x;
   M.redrawImmediately();
  }

  Crop.flipV = function(){
   Cp.mirror = !Cp.mirror;
   Cp.transform.y = -Cp.transform.y;
   let r = 0.5-Cp.transform.rotation;
   Cp.transform.rotation = r - Math.floor(r);
   M.redrawImmediately();
  }

  return Crop;

 });

})();
