(function(){
 'use strict';

 const Ld = sLoader;

 Ld.module('graph',function(){

  const Graph = {};
  const U = Ld.use('utility');

  const colorTbl = [
   '#900', '#990', '#090', '#099', '#009', '#909',
  ];

  const lineStyle = [
   '',
   '2',
   '5 1',
   '2 2 5 2',
  ];


  const padT = 10;
  const padB = 22;
  const padL = 12; // %
  const padR = 28;

  function autoRange(min,max){
   // max > min
   if ( min > max ) {
    console.log('hoge');
    throw('fatal');
   }
   if ( min == max ) {
    if ( min == 0 ) {
     min = 0; max = 1;
    } else if ( min < 0 ) {
     max = 0;
    } else {
     min = 0;
    }
   }
   let step = (max-min)/4;
   let exp = Math.floor(Math.log(step)/Math.log(10));
   let m = step / Math.pow(10,exp);
  //console.log(min,max,step,exp,m);
   if ( m >= 5 ) {
    m = 5;
   } else if ( m >= 2 ) {
    m = 2;
   } else {
    m = 1;
   }
   step = m*Math.pow(10,exp);
   let r = {};
   r.min = Math.floor(min/step)*step;
   r.max = Math.ceil(max/step)*step;
   r.step = step;
  // r.ngrid = Math.round((r.max-r.min)/step);
   return r;
  }


  // min, max
  function adjustRange(p){
   let q = {};
   if ( !p.data || !p.data[0] || !p.data[0][0] ) {
    return;
   }
   let xmin = p.data[0][0][0];
   let ymin = p.data[0][0][1];
   if ( xmin === undefined || ymin === undefined ) {
    return;
   }
   let xmax = xmin;
   let ymax = ymin;
   for( let i = 0; i < p.data.length; i++ ) {
    let d = p.data[i];
    for( let j = 0; j < d.length; j++ ) {
     if ( d[j] !== undefined ) {
      if ( d[j][0] < xmin ) { xmin = d[j][0]; }
      if ( d[j][0] > xmax ) { xmax = d[j][0]; }
      if ( d[j][1] < ymin ) { ymin = d[j][1]; }
      if ( d[j][1] > ymax ) { ymax = d[j][1]; }
     }
    }
   }
  // xmax > xmin

   if ( p.x.min !== undefined ) { xmin = p.x.min; }
   if ( p.x.max !== undefined ) { xmax = p.x.max; }
   if ( p.y.min !== undefined ) { ymin = p.y.min; }
   if ( p.y.max !== undefined ) { ymax = p.y.max; }

   let r = autoRange(xmin,xmax);
   if (p.x.log){
    throw('mada');
   } else {
    q.xmin = r.min;
    q.xmax = r.max;
   }
   q.xstep = r.step;
   r = autoRange(ymin,ymax);
   if (p.y.log){
    throw('mada');
   } else {
    q.ymin = r.min;
    q.ymax = r.max;
   }
   q.xrange = q.xmax - q.xmin;
   q.yrange = q.ymax - q.ymin;
   q.ystep = r.step;
   return q;
  }


  /*
   box:{
    width:      *
    height:     *
   }
   {
   data: [
    [ [x,y], [x,y] ... ]
    ...
   ]
   x:{
    min:        - 
    max:        -
    title:      -
   }
   y:{
    min:        -
    max:        -
    title:      -
   }
   legend{
    "text1",
    "text2",
    ...
   }


  */
  Graph.multipleLineChart = function(p){
let svg = '';
try{
   let q = adjustRange(p);
   if ( !q ) {
    return '';
   }
   let clipPathId = 'cp1';
   let box = p.box;
   let crect = {}; // content rect
   crect.width = box.width*(1-(padL+padR)/100);
   crect.height = box.height*(1-(padT+padB)/100);
   crect.x = box.width*(padL/100);
   crect.y = box.height*(padT/100);

   svg = `<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
   <svg viewBox="0 0 ${box.width} ${box.height}" width="100%" height="100%"
    xmlns="http://www.w3.org/2000/svg"
    xmlns:xlink="http://www.w3.org/1999/xlink" >
    <defs>
     <clipPath id="${clipPathId}" clipunits="userSpaceOnUse">
      <rect x="0" y="0" width="${crect.width}" height="${crect.height}" />
     </clipPath>
    </defs>
    `;
   svg += `<rect x="${crect.x}" y="${crect.y}"
    width="${crect.width}" height="${crect.height}"
    stroke="black" stroke-width="1" fill="none" />`;


   // axis title
   // major tick mark 線
   // axis label      数値
   function plotYAxis(){
    let tm = '';
    let al = '';
    let x = crect.x;
    let exp = Math.floor(Math.log(q.ystep)/Math.log(10));
    function plotAxisLabel(v){
     let text = U.roundFixed(v,-exp);
     let metrics = U.measureText(text,{
      fontFamily:'monospace', fontSize:'10px',
     });
     let y = (1-(v-q.ymin)/(q.ymax-q.ymin))*crect.height + crect.y;
     al += `<text x="${crect.x-metrics.width-5-4}" y="${y}"
      >${text}</text>`;
    }
    function plotTickMark(v){
     let y = (1-(v-q.ymin)/(q.ymax-q.ymin))*crect.height + crect.y;
     tm += ` M ${x-5} ${y} h ${5}`;
    }
    plotAxisLabel(q.ymin);
    plotTickMark(q.ymin);
    let y0 = Math.ceil((q.ymin+q.ystep*0.5)/q.ystep)*q.ystep;
    for( let i = 0; y0+i*q.ystep < q.ymax-q.ystep*0.5 ; i++ ){
     let v = y0+i*q.ystep;
     plotAxisLabel(v);
     plotTickMark(v);
    }
    plotAxisLabel(q.ymax);
    plotTickMark(q.ymax);
    svg += `<g font-family="monospace" font-size="10" >${al}</g>
     <path d="${tm}" stroke="black" stroke-width="1" />`;

    let metrics = U.measureText(U.roundFixed(q.ymin,-exp),{
     fontFamily:'monospace', fontSize:'10px',
    });
    let labelw = metrics.width;
    metrics = U.measureText(U.roundFixed(q.ymax,-exp),{
     fontFamily:'monospace', fontSize:'10px',
    });
    if ( metrics.width > labelw ) {
     labelw = metrics.width;
    }
    if ( p.y.title ) {
     let metrics = U.measureText(p.y.title,{
      fontSize:"16px", fontFamily:"sans-serif"
     });
     x = crect.x-labelw-5-4-24;
     let y = crect.y+crect.height-(crect.height-metrics.width)/2;
     svg += `<text x="${x}" y="${y}" transform="rotate(-90,${x},${y})"
      font-size="16" font-family="sans-serif"
      >${p.y.title}</text>`;
    }
   }

   function plotXAxis(){
    let tm = '';
    let al = '';
    let y = crect.y+crect.height;
    let exp = Math.floor(Math.log(q.xstep)/Math.log(10));
    function plotAxisLabel(v){
     let text = U.roundFixed(v,-exp);
     let metrics = U.measureText(text,{
      fontFamily:'monospace', fontSize:'10px',
     });
     let x = (v-q.xmin)/(q.xmax-q.xmin)*crect.width + crect.x;
     al += `<text x="${x}" y="${y+metrics.width+5+4}"
        transform="rotate(-90,${x},${y+metrics.width+5+4})"
      >${text}</text>`;
    }
    function plotTickMark(v){
     let x = (v-q.xmin)/(q.xmax-q.xmin)*crect.width + crect.x;
     tm += ` M ${x} ${y} v ${5}`;
    }
    plotAxisLabel(q.xmin);
    plotTickMark(q.xmin);
    let x0 = Math.ceil((q.xmin+q.xstep*0.5)/q.xstep)*q.xstep;
    for( let i = 0; x0+i*q.xstep < q.xmax-q.xstep*0.5 ; i++ ){
     let v = x0+i*q.xstep;
  //console.log(i,v);
     plotAxisLabel(v);
     plotTickMark(v);
    }
    plotAxisLabel(q.xmax);
    plotTickMark(q.xmax);
  //console.log(al);
    svg += `<g font-family="monospace" font-size="10" >${al}</g>
     <path d="${tm}" stroke="black" stroke-width="1" />`;

    let metrics = U.measureText(U.roundFixed(q.xmin,-exp),{
     fontFamily:'monospace', fontSize:'10px',
    });
    let labelw = metrics.width;
    metrics = U.measureText(U.roundFixed(q.xmax,-exp),{
     fontFamily:'monospace', fontSize:'10px',
    });
    if ( metrics.width > labelw ) {
     labelw = metrics.width;
    }
    if ( p.x.title ) {
     let metrics = U.measureText(p.x.title,{
      fontSize:"16px", fontFamily:"sans-serif"
     });
     y = crect.y+crect.height+labelw+5+4+24+metrics.baselineOffset;
     let x = crect.x+(crect.width-metrics.width)/2;
     svg += `<text x="${x}" y="${y}" font-size="16" font-family="sans-serif"
      >${p.x.title}</text>`;
    }
   }

   plotXAxis();
   plotYAxis();

   function plotLegend(){
    if ( p.legend ) {
     let x = crect.x + crect.width + 32;
     let y = crect.y;
     for( let i = 0; i < p.legend.length; i++ ){
      let color = colorTbl[i % colorTbl.length];
      let style = lineStyle[Math.floor(i/colorTbl.length) % lineStyle.length];
      svg += `<path d="M ${x} ${y} h 40"
       stroke="${color}" stroke-dasharray="${style}" stroke-width="1" />
       <text x="${x+52}" y="${y+6}" font-size="12" font-family="sans-serif"
       >${p.legend[i]}</text>`;
      y += 16;
     }
    }
   }
   plotLegend();

   svg += `<g transform="translate(${crect.x} ${crect.y})">
    <g clip-path="url(#${clipPathId})">`;

   let k = 0;
   for( let i = 0; i < p.data.length; i++ ) {
    let data = p.data[i];
    let d = '';
    let color = colorTbl[k % colorTbl.length];
    let style = lineStyle[Math.floor(k/colorTbl.length) % lineStyle.length];
  //console.log('style',style);
    for( let j = 0; j < data.length; j++ ) {
     if ( data[j] !== undefined ) {
      let x = data[j][0];
      let y = data[j][1];
      if ( p.x.log ) { x = Math.log(x); }
      if ( p.y.log ) { y = Math.log(y); }
      x = (x-q.xmin)/q.xrange*crect.width;
      y = (1-(y-q.ymin)/q.yrange)*crect.height;
      d += ` ${j == 0? 'M': 'L'} ${x} ${y}`;
      svg += `<rect x="${x-1}" y="${y-1}" width="3" height="3" fill="${color}" />`;
     }
    }
    svg += `<path d="${d}" stroke="${color}" stroke-width="1" fill="none"
     stroke-dasharray="${style}" />`;
    k++;
   }
   svg += '</g></g>';
   svg += '</svg>';
}
catch(e){
 console.log(e);
 svg = '';
}
   return svg;

  }

  return Graph;
 });
})();
