(function(){
 'use strict';

 const Ld = sLoader;
 Ld.module('main',function(){
  const M = {};
  this.body = M;

  const U = Ld.use('utility');
  const Graph = Ld.use('graph');
  const ExifReader = Ld.use('exifReader');
  const FpDetect = Ld.use('fp-detect');


  const graphViewWidth = 800;
  const graphViewHeight = 400;
  const fileViewWidth = 800;

  let srcFiles = [];
  let processCounter = 0;
  let processStatus;
  let lastSelect;


//
  M.log = console.log;
  M.error = console.log;

  function $(id){
   return document.getElementById(id);
  }


  M.main = function(){
   $('menuButton').addEventListener('click',function(){
    let button = $('menuButton');
    let menu = $('menu');
    U.setStyle(menu.style,{
     display: 'block',
    });
   });

   $('menu').addEventListener('mouseleave',function(ev){
    hideMenu(ev);
   });

   $('addFiles').addEventListener('click',function(ev){
    hideMenu(ev);
    U.openFileP({accept:'image/*',multiple:true,name:'files[]'})
    .then(files=>{
     addFiles(files);
    });
   });
   $('import').addEventListener('click',function(ev){
    hideMenu(ev);
    U.openFileP({accept:'application/json,text/plain',name:'files[]'})
    .then(files=>{
     importData(ev.target.files[0]);
    });
   });

   $('exportButton').addEventListener('click',function(ev){
    hideMenu(ev);
    exportData();
   });

   $('clearButton').addEventListener('click',function(ev){
    hideMenu(ev);
    clearSelection();
   });

   $('removeButton').addEventListener('click',function(ev){
    hideMenu(ev);
    removeSelectedFile();
   });

   $('startButton').disabled = true;
   $('startButton').addEventListener('click',function(ev){
    hideMenu(ev);
    profilerStart();
   });

   $('zoomCurve').addEventListener('change',function(ev){
    updateGraph();
   });

   window.addEventListener('resize',onresize);

   let dropBox = document.getElementById('fileList');
   dropBox.addEventListener('dragover', onDragOver, false );
   dropBox.addEventListener('drop', onDrop, false );

   function disable(ev){
    ev.preventDefault();
   }
   window.addEventListener('dragover', disable, false );
   window.addEventListener('dragenter', disable, false );
   window.addEventListener('drop', disable, false );

   updateGraph();
   updateInfo();
   onresize();
  };



  function hideMenu(ev){
    ev.stopPropagation();
    $('menu').style.display = 'none';
  }


  function fitImage(el){
   let div = el.parentNode;
   let w = $('imageView').clientWidth*0.6;
   let scale = w/el.width;
   let h = el.height*scale;
   U.setStyle(el.style,{
    transformOrigin:`top left`,
    transform:`scale(${scale})`,
   });
   U.setStyle(div.style,{
    width:w+'px', height:h+'px'
   });
  }

  function resizeEdgeView(){
   let images = $('imageView').getElementsByTagName('canvas');
   for( let i = 0; i < images.length; i++ ) {
    fitImage(images[i]);
   }
  }

  function resizePreview(){
   let images = $('imageView').getElementsByTagName('img');
   for( let i = 0; i < images.length; i++ ) {
    fitImage(images[i]);
   }
  }

  function onresize(){
   // graph
   let pane, view, scale;
   pane = document.getElementById('graphPane');
   view = document.getElementById('graphView');
   scale = pane.clientWidth/graphViewWidth;
   U.setStyle(view.style,{
    fontSize:(12*scale)+'pt',
   });
   resizeEdgeView();
   resizePreview();
  }


  function getIndex(tr){
   let a = $('fileListBody').getElementsByTagName('tr');
   let i = 0;
   for( ; i < a.length && a[i] !== tr; i++ ) {}
   if ( a[i] !== tr ) return;
   return i;
  }


  function fnameOnClick(ev){
   let i = getIndex(ev.target.parentNode);
   if ( i !== undefined ) {
     let r = srcFiles[i];
     //
     let el = document.getElementById(escapeId('i'+r.name));
     if ( el ) {
      $('imageView').scrollTop = el.offsetTop;
     }
   }
  }

  function checkboxOnClick(ev){
   ev.stopPropagation();
   ev.preventDefault();
   let i = getIndex(ev.target.parentNode);

// lastSelectはクリアしないと
   if ( i !== undefined ) {
    if ( ev.shiftKey && lastSelect !== undefined ) {
     let start = Math.min(i,lastSelect);
     let end = Math.max(i,lastSelect);
     let value = srcFiles[lastSelect].selected;
     for( let j = start; j <= end; j++ ) {
      srcFiles[j].selected = value;
      updateRecord(srcFiles[j]);
     }
    } else {
     let f = srcFiles[i];
     f.selected = !f.selected;
     updateRecord(f);
    } 
    lastSelect = i;
    updateInfo();
   }
  }


  function trOnClick(ev){
   let td = ev.target;
   if ( td.className === 'fname' ) {
    fnameOnClick(ev);
   }
   else if ( td.className === 'check' ) {
    checkboxOnClick(ev);
   }
  }


// U.hex削除...
  function escapeId(s){
   s = encodeURIComponent(s);
   s = s.replace(/([^a-zA-Z0-9%\-])/g,
    (s,p1)=>'%'+U.hex(p1.charCodeAt(0),2,'0')
   );
   return s.replace(/%/g,'_');
  }


// 
  function onLoadImage(f,image,after){


 // 下は別に

   const cropH = 50;    // (%)
   const cropV = 10;

   const PREVIEW_WIDTH = 480;
   const PREVIEW_HEIGHT = 120;

   //const rBlurH = 1;
   // 横は値を使って補完するべき

   let block = document.createElement('div');
   block.id = escapeId('i'+f.name);

   block.innerHTML = `<h3>${f.name}</h3>`;

   let div = document.createElement('div');
   div.appendChild(image);
   block.appendChild(div);
   fitImage(image);

   div = document.createElement('div');
   let canvas = document.createElement('canvas');
   div.appendChild(canvas);
   block.appendChild(div);

   // 画像の中央を取り出す
   let w = Math.floor(image.width*(cropH/100));
   let h = Math.floor(image.height*(cropV/100));

   canvas.width = w;
   canvas.height = h;
   fitImage(canvas);

   $('imageView').insertBefore(block,
    after? document.getElementById(escapeId('i'+after.name)): null
   );
   $('imageView').scrollTop = block.offsetTop;

   let ctx = canvas.getContext('2d');
   ctx.save();
   ctx.translate(w/2,h/2);
   if ( !f.vertical ) {
    ctx.rotate(-270/180*Math.PI);
   }
   ctx.translate(-image.width/2,-image.height/2);
   ctx.drawImage(image,0,0);
   ctx.restore();
   
   let imagedata = ctx.getImageData(0,0,w,h);


// ファイルごとに
// imageBuf
// 再利用する
   let imgGray = new Array(w*h);

   rgbToGray(imagedata.data,imgGray,w,h);

   let r = FpDetect.detect(imgGray,w,h,f.pitch,image.width);

   if ( r && r.edgeList ) {

    let edgeList = r.edgeList;

 //   grayToRGB(imgGray,imagedata.data,w,h);
    grayToRGB(r.blur,imagedata.data,w,h);
    ctx.putImageData(imagedata,0,0);

 // スクリーンで。
    ctx.save();
    ctx.fillStyle = 'rgba(0,0,0,0.6)';
 //   ctx.fillRect(0,0,w,h);
    ctx.restore();

 // svgの方がいい?
    for( let i = 0; i < edgeList.length; i++ ) {
     let edge = edgeList[i].list;
     let y = edgeList[i].y;
     ctx.save();
     ctx.beginPath();
     ctx.strokeStyle = 'rgb(128,255,255)';
     ctx.lineWidth = 5;
     for( let i = 0; i < edge.length; i++ ) {
      let x = edge[i];
      ctx.moveTo(x,y-6);
      ctx.lineTo(x,y+6);
     }
     ctx.stroke();
     ctx.restore();
    }
   }
   if ( r && r.fov ) {
    f.fov = r.fov;
    f.status = 'complete';
   }
   else {
// あっちこっちでエラー
    f.status = 'error';
   }

   updateInfo();
   updateRecord(f);
   hilightRecord(f,false);
   updateGraph();
  
  }


  function rgbToGray(s,t,w,h){
   for( let y = 0; y < h; y++ ) {
    for( let x = 0; x < w; x++ ) {
     let i = (y*w+x)*4;
     t[y*w+x] = Math.max(s[i+0],s[i+1],s[i+2]);
    }
   }
  }


  function grayToRGB(s,t,w,h){
   for( let y = 0; y < h; y++ ) {
    for( let x = 0; x < w; x++ ) {
     let g = s[y*w+x];
     t[(y*w+x)*4+0] = g;
     t[(y*w+x)*4+1] = g;
     t[(y*w+x)*4+2] = g;
    }
   }
  }


  function makeRecord(r){
   let r0 = srcFiles[0];
   let status = 
    r.status == 'complete'? '○':
    r.distance === undefined || r.pitch === undefined ||
     r.key != r0.key || r.status == 'error'
    ? `<span class="red">×</span>`
    : '';

   return `<td class="check">${r.selected? '\u2713':''}</td>
    <td class="status">${status}</td>
    <td class="fname">${r.name}</td>
    <td class="distance">${r.distance!==undefined? r.distance:''}</td>
    <td class="pitch">${r.pitch? r.pitch: '' }</td>
    <td class="zsc">${r.zsc||''}</td>
    <td class="fsc">${(r.fsc||0).toFixed(2)}</td>
    <td class="fov">${
     r.fov !== undefined? U.roundRelative(r.fov,4): ''
    }</td>`;
  }

  /*
  */
  function showFileList(){

   let html = '<table class="fileListTbl"><tbody>';
   for( let i = 0; i < srcFiles.length; i++ ) {
    let f = srcFiles[i];
    html += `
     <tr class="filerec" id="${escapeId('r'+f.name)}">
      ${makeRecord(f)}
     </tr>`;
   }
   html += '</tbody></table>';
   let el = document.getElementById('fileListBody');
   el.innerHTML = html;

   let a = el.getElementsByTagName('tr');
   for( let i = 0; i < a.length; i++ ) {
    a[i].addEventListener('mousedown', trOnClick,false);
   }

  }

  // 1origin
  function getNthRecord(table,n){
   return table.getElementsByTagName('tr')[n-1];
  }

  function updateRecord(f){
   let table = $('fileListBody').firstChild;
   let tr = getNthRecord(table,f.index+1);
   tr.innerHTML = makeRecord(f);
  }

  function hilightRecord(f,sw){
   let table = $('fileListBody').firstChild;
   let tr = getNthRecord(table,f.index+1);
   tr.className = 'filerec'+ (sw? ' hilight': '');

   let view = $('fileListBody');

   if ( tr.offsetTop < view.scrollTop ) {
    view.scrollTop = tr.offsetTop;
   } else if ( tr.offsetTop+tr.clientHeight
     > view.scrollTop+view.clientHeight ) {
    view.scrollTop = tr.offsetTop+tr.clientHeight-view.clientHeight;
   } else {
   }

  }


  function delayP(ms){
   return new Promise((resolve, reject) => {
    setTimeout(resolve,ms);
   });
  }


// start/stop
  function profilerStart(){
   if ( !processStatus ) {
    $('startButton').innerHTML = 'Stop Analizing';
    processStatus = true;
    analizeNext();
   } else {
    $('startButton').innerHTML = 'Start Analizing';
    processStatus = false;
   }
  }


  function analizeNext(){
   (async()=>{
    for( let i = 0; i < srcFiles.length; i++ ) {
     if ( srcFiles[i].status === 'complete' ) continue;
     let f = srcFiles[i];
     let after = srcFiles[i+1];
     hilightRecord(f,true);
     let image = await U.loadImageP(f.file);
     onLoadImage(f,image,after);
     if ( !processStatus ) break;
    }
   })()
   .then(()=>{
    processStatus = false;
   })
   .catch((e)=>{
    console.log(e);
    processStatus = false;
   });
  }


// updateInfo
  function updateInfo(){

   let status;
   if ( srcFiles.length == 0 ) {
    status = `<span class="yellow">Open, Drop image files or Import</span>`;
   } else {
    let q = srcFiles[0];
    let completed = true;
    for( let i = 0; i < srcFiles.length; i++ ) {
     let r = srcFiles[i];
     if ( r.status != 'complete' &&
       ( r.distance === undefined || r.pitch === undefined ) ) {
      status = `<span class="red">File name must include distance and pitch information. (e.g. d00-p0_46617-01.jpg)</span>`;
      break;
     }
     if ( r.key === undefined ) {
      status = `<span class="red">Unknown camera system.</span>`;
      break;
     }
     if ( r.key != q.key ) {
      status = `<span class="red">Camera systems are not unified.</span>`;
      break;
     }
     // fov error
     if ( r.status == 'error' ) {
      status = `<span class="red">FOV analizer failed.</span>`;
      break;
     } else if ( r.status != 'complete' ) {
      completed = false;
     }
    }
    if ( !status ) {
     if ( !completed ) {
      status = `<span class="yellow">Ready to analize.</span>`;
     } else {
      status = `<span class="green">Ready to  export.</span>`;
     }
    }
   }
   $('status').innerHTML = status;

   //
   if ( srcFiles.length > 0 ) {
    let r = srcFiles[0];
    if ( $('description').value === '' ) {
     let desc = U.htmlEscape(r.bodyType||r.cameraType);
     if ( r.lensType ) { desc += ' + ' + U.htmlEscape(r.lensType); }
     if ( r.extenderType ) { desc += ' + ' + U.htmlEscape(r.extenderType); }
     if ( r.mode ) { desc += ' + ' + U.htmlEscape(r.mode); }
     $('description').value = desc; 
    }
   }

   // 
   let r = srcFiles[0];
   for( let i = 0; i < srcFiles.length; i++ ) {
    if ( srcFiles[i].selected ) {
     r = srcFiles[i];
     break;
    }
   }
   if ( r ) {
    let camera = r.bodyType? r.bodyType: r.cameraCode;
    $('camera').innerHTML = camera?
     `${camera} (${U.htmlEscape(r.cameraType)})`: '';
    $('lens').innerHTML = r.lensCode?
     `${r.lensCode} (${U.htmlEscape(r.lensType||'')})`: '';
    $('extender').innerHTML = r.extenderCode?
     `${r.extenderCode} (${U.htmlEscape(r.extenderType||'')})`: '';
    $('mode').innerHTML = r.modeCode?
     `${r.modeCode} (${U.htmlEscape(r.mode||'')})`: '';
   }
  }

  function onLoadExif(f,exif){
   f.zsc = exif.ZoomStepCount||0;
   f.fsc = exif.FocusStepCount||0;
   f.fscinf = exif.FocusStepInfinity || 0;
   f.exif = true;
   f.makerCode = exif._MakerCode;
   f.cameraCode = exif._CameraCode;
   f.cameraType = exif.CameraType;
   f.bodyType = exif._BodyType;
   f.lensCode = exif._LensCode;
   f.lensType = exif.LensType || f.lensCode;
   f.extenderCode = exif._ExtenderCode;
   f.extenderType = exif.Extender || f.extenderCode;
   f.modeCode = exif._ModeCode;
   f.mode = exif._Mode || f.modeCode;
   f.key = 
    [f.makerCode,f.cameraCode,f.lensCode,f.extenderCode,f.modeCode].join('-');

   updateInfo();
   updateRecord(f);
   hilightRecord(f,false);
  }


  function loadExif(){
   (async()=>{
    for( let i = 0; i < srcFiles.length; i++){
     if ( srcFiles[i].exif ) continue;
     let f = srcFiles[i];
     let exif = await ExifReader.readP(f.file)
     onLoadExif(f,exif);
    }
    return true;
   })()
   .then(()=>{
    updateGraph();
   })
   .catch((e)=>{
    console.log(e);
    updateGraph();
   });
  }


  function clearSelection(){
   for( let i = 0; i < srcFiles.length; i++ ) {
    let r = srcFiles[i];
    r.selected = false;
   }
   lastSelect = undefined;
   updateInfo();
   showFileList();
  }


  function removeSelectedFile(){
   let newList = [];
   for( let i = 0; i < srcFiles.length; i++ ) {
    let r = srcFiles[i];
    if ( r.selected ) {
     let el = document.getElementById(escapeId('i'+r.name));
     if ( el ) {
      $('imageView').removeChild(el);
     }
    } else {
     r.index = newList.length;
     newList.push(r);
    }
   }
   srcFiles = newList;
   lastSelect = undefined;
   updateInfo();
   showFileList();
  }


//
  function convertData(profile,fields){
   // try!!
   if ( profile.fields === undefined ) {
    // 配列でない場合とかも!!
    console.log('invalid profile data');
    return;
   }
   let idx0 = {};
   for( let i = 0; i < profile.fields.length; i++ ) {
    idx0[profile.fields[i]] = i;
   }
   let idx = [];
   for( let i = 0; i < fields.length; i++ ) {
    idx[i] = idx0[fields[i]];
    if ( idx[i] === undefined ) {
     // メッセージ
     console.log('invalid profile data');
     return;
    }
   }
   let data = [];
   for( let i = 0; i < profile.data.length; i++ ) {
    let r = {};
    for( let j = 0; j < fields.length; j++ ) {
     r[fields[j]] = profile.data[i][idx[j]];
    }
    data[i] = r;
   }
   return data;
  }


// 保留
  function importData(file){
   console.log('import',file);
   U.readTextFile(file,function(text){
    if ( !text ) {
     //
     return;
    }
    let profile = JSON.parse(text);
    if ( profile.signature != 'Scabar FOV Profile' ) {
     //
     console.log('signature');
     return;
    }

    let data;
    switch(profile.type){
     case 'fixed-magnification':
      data = [ { fov:profile.data, name:'-' } ];
      break;
     case 'fixed-focal-length':
      data = convertData(profile,['fsc','fov']);
      for( let i = 0; i < data.length; i++ ) {
       data[i].name = `d${data[i].fsc}`;
      }
      break;
     case 'zoom':
      data = convertData(profile,['distance','zsc','fsc','fov']);
      for( let i = 0; i < data.length; i++ ) {
       data[i].name = `d${data[i].distance}-z${data[i].zsc}`;
      }
      break;
    }

    $('description').value = profile.description;

    let key = [profile.maker,profile.camera,profile.lens,
      profile.extender,profile.mode].join('-');
    let index = {};
    for( let i = 0; i < srcFiles.length; i++ ) {
     let name = srcFiles[i].name;
     index[name] = srcFiles[i];
    }
    for( let i = 0; i < data.length; i++ ) {
     let r = data[i];
     let s = {};
     s.status = 'complete';
     s.name = r.name;
     s.distance = r.distance;
     s.fsc = r.fsc;
     s.fsc = r.fscinf;
     s.zsc = r.zsc;
     s.fov = r.fov;
     s.key = key;
     s.makerCode = profile.maker;
     s.cameraCode = profile.cameraCode;
     s.cameraType = profile.cameraType;
     s.bodyType = profile.bodyType;
     s.lensCode = profile.lensCode;
     s.lensType = profile.lensType;
     s.extenderCode = profile.extenderCode;
     s.extenderType = profile.extenderType;
     s.modeCode = profile.mode;
//
     s.exif = true;
     index[s.name] = s;
    }
    let list = Object.keys(index).map(function(name){return index[name];});

    srcFiles = list.sort((a,b)=>
     a.zsc !== b.zsc? a.zsc - b.zsc: a.distance - b.distance
    );

    if ( srcFiles.length > 0 ) {
     for( let i = 0; i < srcFiles.length; i++ ) {
      srcFiles[i].index = i;
     }
     updateInfo();
     showFileList();
     updateGraph();
    }
   });
  }


  function addFiles(files){

   let index = {};
   for( let i = 0; i < srcFiles.length; i++ ) {
    let key = srcFiles[i].name;
    index[key] = srcFiles[i];
   }

   for (let i = 0; i < files.length; i++) {
    let f = files[i];
    if ( f.type.match(/image\/*/) ) {
     let r = {};
     r.file = f;
     r.name = f.name;

     // -d
     let p = f.name.match(/(^|-)d([\d_]+)/i);
     if ( p ) {
      let v = parseFloat(p[2].replace(/_/,'.'));
      if ( v !== undefined ) {
       r.distance = v;
      }
     }

     // -z
     p = f.name.match(/(^|-)z([\d_]+)/i);
     if ( p ) {
      let v = parseFloat(p[2].replace(/_/,'.'));
      if ( v !== undefined ) {
       r.zoom = v;
      }
     }

     // -p
     p = f.name.match(/(^|-)p([\d_]+)/i);
     if ( p ) {
      let v = parseFloat(p[2].replace(/_/,'.'));
      if ( v !== undefined ) {
       r.pitch = v;
      }
     }

     // -v 
     p = f.name.match(/(^|-)v/i);
     if ( p ) {
      r.vertical = true;
     }

// inf.
     // -f 
     p = f.name.match(/(^|-)f([\d_]+)/i);
     if ( p ) {
      let v = parseFloat(p[2].replace(/_/,'.'));
      if ( v !== undefined ) {
       r.fov = v;
       r.status = 'complete';
      }
     }
     let key = r.name;
     index[key] = r;

    }
   }

   let list = Object.keys(index).map(function(key){return index[key];});

   srcFiles = list.sort((a,b)=>
     a.distance!=b.distance? a.distance-b.distance: a.zoom-b.zoom
   );
   if ( srcFiles.length > 0 ) {
    for( let i = 0; i < srcFiles.length; i++ ) {
     srcFiles[i].index = i;
    }
    showFileList();
    processCounter = 0;
    loadExif();
    $('startButton').disabled = false;
     // exifのあと!
   }
  }


  function onDrop(ev) {
   ev.stopPropagation();
   ev.preventDefault();
   addFiles(ev.dataTransfer.files);
  }


  function onDragOver(ev) {
   ev.stopPropagation();
   ev.preventDefault();
   ev.dataTransfer.dropEffect = 'copy';
  }


  function escape(s){
   if ( s == null ) return '';
   return encodeURIComponent(s)
    .replace(/([^a-zA-Z0-9%\_\-\ ])/g,
     c=>'%'+('0'+c.charCodeAt(0).toString(16)).substr(-2)
    );
  }

/*
  function makeKeyMain(profile){
   return ['maker', 'bodyType', 'lensCode', 'extenderCode', 'mode']
    .map(a=>escape(profile[a])).join('+');
  }

  function makeKeySub(profile){
   return escape(profile.attachment);
  }
*/


  function exportData(){
   let profile = makeProfile();
   let json = '';

   function put(key){
    if ( profile[key] ) {
     json += ` "${key}": ${JSON.stringify(profile[key])},\n`;
    }
   }

   if ( profile ) {

    json += '{\n';

    let tags = [
     'maker', 'cameraCode', 'cameraType', 'bodyType',
     'lensCode', 'lensType', 'extenderCode', 'extenderType',
     'attachment', 'mode', 'time', 'description', 'type'
    ];
    for( let i = 0; i < tags.length; i++ ) {
     put(tags[i]);
    }

    switch(profile.type){

    case 'fixed-magnification':
     let r = profile.data[0];
     json += ` "data": ${r.v.toFixed(3)}\n`;
     break;

    case 'fixed-focal-length':
     json += ` "fields": [ "fsc", "fov" ],\n "data": [\n`;
     for( let i = 0; i < profile.data.length; i++ ) {
      let r = profile.data[i];
      if ( i != 0 ) { json += ',\n'; }
      json += `  [ ${r.fsc}, ${r.fov.toFixed(3)} ]`;
     }
     json += '\n ]\n';
     break;

    case 'zoom':
// fscinf
     put('zoomCurve');
     json += ` "fields": [ "zsc", "distance", "fsc", "fscinf", "fov" ],\n "data": [\n`;
     for( let i = 0; i < profile.data.length; i++ ) {
      let r = profile.data[i];
      if ( i != 0 ) { json += ',\n'; }
      json += `  [ ${r.zsc}, ${r.distance}, ${r.fsc}, ${r.fscinf}, ${r.fov.toFixed(3)} ]`;
     }
     json += '\n ]\n';
     break;
    }
    json += '}\n';


    function esc(s){
     s = s.replace(/[\s\-]/g,'_').replace(/\./g,'p');
     s = encodeURIComponent(s);
     return s.replace(/([^a-zA-Z0-9_%])/g,function(s,p1){
      return '%'+U.hex(p1.charCodeAt(0),2,'0');
     });
    }

    let maker      = esc(profile.maker||'');
    let camera     = esc(profile.bodyType||profile.cameraType||'');
    let lens       = esc(profile.lensType||profile.lensCode||'');
    let extender   = esc(profile.extenderType||profile.extenderCode||'');
    let attachment = esc(profile.attachment||'');
    let mode       = esc(profile.mode||'');

    let fileName = `${maker}-${camera}-${lens}-${extender}-${attachment}-${mode}.json`;

//    json = json.replace(/\u2028/g, '\\u2028').replace(/\u2029/g,'\\u2029');
    U.downloadText(fileName,json,'application/javascript');
   }
  }




//////////////////////////////////////////////////////////////////////


  // format
  // export
  function makeProfile(){
   let profile = {};
   profile.description = $('description').value;
   let time = new Date();
   let month = U.dec(time.getMonth  ()+1,2,'0');
   let date  = U.dec(time.getDate   ()  ,2,'0');
   let hour  = U.dec(time.getHours  ()  ,2,'0');
   let min   = U.dec(time.getMinutes()  ,2,'0');
   let sec   = U.dec(time.getSeconds()  ,2,'0');
   profile.time = 
    `${time.getFullYear()}${month}${date}T${hour}${min}${sec}`;

   profile.attachment = $('attachment').value;

   if ( srcFiles.length > 0 ) {
    let r = srcFiles[0];
    profile.maker = r.makerCode;
    profile.cameraCode = r.cameraCode;
    profile.cameraType = r.cameraType;
    profile.bodyType = r.bodyType;
    profile.lensCode = r.lensCode;
    profile.lensType = r.lensType;
    profile.extenderCode = r.extenderCode;
    profile.extenderType = r.extenderType;
    profile.mode = r.modeCode;
   }

   let list = [];
   let type = '';
   let zoom;
   if ( srcFiles.length ) {
    let r = srcFiles[0];
    zoom = r.zsc;
   }
   for( let i = 0; i < srcFiles.length; i++ ) {
    let f = srcFiles[i];
    if ( f.distance !== undefined &&
      f.zsc !== undefined &&
      f.fsc !== undefined ) {
      // グラフを出すためfovはundefinedでも良い。
      // エラーは先に判定する
// 
     let rec = { distance: f.distance, zsc: f.zsc, fsc: f.fsc, fov: f.fov, fscinf: f.fscinf };
     if ( zoom != f.zsc ) { type = 'zoom'; }
     list.push(rec);
    }
   }
   profile.data = list.sort(function(a,b){
    return a.zsc != b.zsc? a.zsc - b.zsc:
     a.distance != b.distance? a.distance - b.distance: 0;
   });
   profile.type = srcFiles.length == 1? 'fixed-magnification':
    type==='zoom'? 'zoom': 'fixed-focal-length';
   if ( type==='zoom' ) {
    profile.zoomCurve = $('zoomCurve').value;
   }
   return profile;
  }


  //
  function updateGraph(){

   let profile = makeProfile();


   function separate(key,dataIn){
    let dataOut = [];
    let key0;
    let j = -1;
    for( let i = 0; i < dataIn.length; i++ ) {
     let r = dataIn[i];
     if ( r[key] != key0 ) {
      key0 = r[key];
      dataOut[++j] = [];
     }
     dataOut[j].push(r);
    }
    return dataOut;
   }


   function fsc_mag(profile){
    let pdata = separate('zsc',profile.data.filter(a=>a.fov !== undefined)
      .sort((a,b)=>a.zsc != b.zsc? a.zsc - b.zsc: a.fsc - b.fsc));
    let data = [];
    let legend = [];
    for( let i = 0; i < pdata.length; i++ ) {
     data[i] = [];
     legend[i] = `z = ${pdata[i][0].zsc}`;
     for( let j = 0; j < pdata[i].length; j++ ) {
      let r = pdata[i][j];
      data[i][j] = [(r.fsc-r.fscinf),36/r.fov];
     }
    }
    let svg = Graph.multipleLineChart({
     data: data,
     x:{ title:'FSC - FSCInf' },
     y:{ title:'Magnification(35mm equiv.)', min:0},
     legend:legend,
     box:{ width: graphViewWidth, height: graphViewHeight },
    });
    return svg;
   }


   function zsc_fov(profile){
    let pdata = separate('distance',profile.data.filter(a=>a.fov !== undefined)
      .sort((a,b)=>a.distance != b.distance? a.distance - b.distance:
        a.zsc - b.zsc));
    let data = [];
    let legend = [];
    for( let i = 0; i < pdata.length; i++ ) {
     data[i] = [];
     legend[i] = `d = ${pdata[i][0].distance}`;
     for( let j = 0; j < pdata[i].length-1; j++ ) {
      let r = pdata[i][j];
      if ( profile.zoomCurve === 'log' ) { // log
       data[i][j] = [r.zsc,Math.log(36/r.fov)/Math.log(10)];
      } else { // linear
       data[i][j] = [r.zsc,36/r.fov];
      }
     }
    }
    let svg = Graph.multipleLineChart({
     data: data,
     x:{ title:'Zoom Step Count'},
     y:{ title:profile.zoomCurve === 'log'? 'Log10(Mag35)': 'Mag35' },
     box:{ width: graphViewWidth, height: graphViewHeight },
     legend:legend,
    });
    return svg;
   }


   // ZSC - FSC 
   function zsc_fsc(profile){
    let pdata = separate('distance',profile.data
      .sort((a,b)=>a.distance != b.distance? a.distance - b.distance:
        a.zsc - b.zsc));
    let data = [];
    let legend = [];
    for( let i = 0; i < pdata.length; i++ ) {
     data[i] = [];
     legend[i] = `d = ${pdata[i][0].distance}`;
     for( let j = 0; j < pdata[i].length; j++ ) {
      let r = pdata[i][j];
      data[i][j] = [r.zsc,(r.fsc-r.fscinf)];
     }
    }
    let svg = Graph.multipleLineChart({
     data: data,
     x:{ title:'Zoom Step Count'},
     y:{ title:'FSC - FSCInf' },
     box:{ width: graphViewWidth, height: graphViewHeight },
     legend: legend,
    });
    return svg;
   }

   let el = $('graphView');
   let t = `
    <h3>1. Magnification(35mm equiv.) vs. Focus Step Count</h3>
    <div class="chart">${fsc_mag(profile)}</div>
    <h3>2. Magnification(35mm equiv.) vs. Zoom Step Count</h3>
    <div class="chart">${zsc_fov(profile)}</div>
    <h3>3. Focus Step Count vs. Zoom Step Count</h3>
    <div class="chart">${zsc_fsc(profile)}</div>
    `;
   el.innerHTML = t;
  };


 });
})();
