(function(){
 'use strict';
 const Ld = sLoader;
 Ld.module('utility',function(){

  const U = {};

 // 
  U.roundFixed           = roundFixed;
  U.roundRelative        = roundRelative;
  U.htmlEscape           = htmlEscape;
  U.correctNumber        = correctNumber;
// U.isArabic
// U.hex

  U.encodeUTF8           = encodeUTF8;
  U.decodeUTF8           = decodeUTF8;

  U.gcd                  = gcd;
  U.dist                 = dist;
// parseDimension

  U.merge                = merge;

// U.getScreenCTM
// U.convertFromScreenCoordinate

  U.setStyle             = setStyle;
  U.styleToString        = styleToString;
  U.fitText              = fitText;
// measureText

  U.splitDataURL         = splitDataURL;
//  U.dataURLToObjectURL   = dataURLToObjectURL;
  U.revokeObjectURL      = revokeObjectURL;
  U.svgTextToObjectURL   = svgTextToObjectURL;

// textToDataURL
// svgToDataURL
// loadImageP
// readTextFile
// downloadText
// openFile

  const M = Ld.ref('main');



  // (123,2) -> '1.23'
  // (123,4) -> '0.0123'
  // (123,-1) -> '1230'
  function insertPoint(ival,pos){
   let str;
   if ( pos <= 0 ) {
    let n = -pos;
    str = Math.abs(ival).toString();
    for( let i = 0; i < n; i++ ) { str += '0'; }
   } else {
    str = Math.abs(ival).toString();
    let len = str.length;
    if ( pos < len ) {
     str = str.substr(0,len-pos) + '.' + str.substr(len-pos);
    } else {
     // if large pos, ...
     for( let i = 0; i < pos-len; i++ ) { str = '0'+str; }
     str = '0.'+str;
    }
   }
   if ( ival < 0 ) { str = '-'+str; }
   return str;
  }


  // round
  // 文字列を返す
// toFixedとかいうのがあるみたい。
  function roundFixed(value,numDigit){
   let base = Math.pow(10,-numDigit);
   return insertPoint(Math.round(value*Math.pow(10,numDigit)),numDigit);
  }


  //
  // (1.2345,3) -> "1.23" 
  // (1234.5,3) -> "1230"
// ->toPrecision
  //
// toPrecition
  function roundRelative(value,significant_figures){
   let str;
   if ( value == 0 ) {
    // 0.00...
    return '0';
   }
   let exponent = Math.floor(Math.log(Math.abs(value))/Math.log(10));
   let numDigit = significant_figures-exponent-1;
   return insertPoint(
    (value<0? -1: 1)*Math.round(Math.abs(value)*Math.pow(10,numDigit)),
    numDigit
   );
  }



  function htmlEscape(str) {
   if ( str === undefined ) {
    str = '(undefined)';
   }
   else if ( str.toString ) {
    str = str.toString();
   }
   return str.replace(/&/g, "&amp;").
    replace(/"/g, "&quot;").
    replace(/</g, "&lt;").
    replace(/>/g, "&gt;");
  }


  function correctNumber(value,min,max,dflt){
   if ( typeof value == 'number' ) { }
   else if ( typeof value == 'string' ) {
    value = parseFloat(value);
   }
   else { value = dflt; }
   if ( isNaN(value) ) { value = dflt; }
   if ( value < min ) { value = min; }
   else if ( value >= max ) { value = max; }
   return value;
  }

  U.isArabic = function(str){
   return str.match(
    /[\u0600-\u06ff\u0750-\u077f\u08a0-\u08ff\ufb50-\ufdff\ufe70-\ufeff]/);
  };


// b, width, padding
  U.hex = function(b,width,pad){
   let s = b.toString(16);
   if ( width && s.length < width ) {
    if ( !pad ) { pad = ' '; }
    s = pad.repeat(width-s.length) + s;
   }
   return s;
  }

  // integer
  U.dec = function(i,width,pad){
   let s = i.toString();
   if ( width && s.length < width ) {
    if ( !pad ) { pad = ' '; }
    s = pad.repeat(width-s.length) + s;
   }
   return s;
  }


 function encodeUTF8(s){
  let t = encodeURI(s);
  let a = [];
  let percent = '%'.charCodeAt(0);
  let j = 0;
  for( let i = 0; i < t.length;){
   let c = t.charCodeAt(i);
   if ( c === percent ) {
    a[j++] = parseInt(t.substr(i+1,2),16);
    i+=3;
   } else {
    a[j++] = c;
    i++;
   }
  }
  let u = new Uint8Array(a.length);
  for( let i = 0; i < a.length; i++ ) {
   u[i] = a[i];
  }
  return u;
 }

 function decodeUTF8(u){
  let s = '';
  for( let i = 0; i < u.length; i++ ) {
   s += '%'+U.hex(u[i],2,'0');
  }
  return decodeURI(s);
 }




  function gcd(x,y){
   if ( x < y ) { let t = x; x = y; y = t; }
   while( y > 0 ) {
    let r = x % y;
    x = y; y = r;
   }
   return x;
  }


  U.parseDimension = function(str){
   let mm;
   let s = str.trim();
   let v = parseFloat(s);
   let unit = '';
   let m = s.match(/([^\s\.0-9]+)$/);
   if ( m ) {
    unit = m[1];
    switch(unit){
    case 'μm':
    case 'µm' :
    case 'um' : mm = v/1000; unit = 'µm'; break;
    case 'mm' : mm = v; break;
    case 'cm' : mm = v*10; break;
    case 'm'  : mm = v*1000; break;
    case 'km' : mm = v*1000000; break;
    default   : unit = ''; break;
    }
   }
   return {
    mm: mm,
    htmlText: v+'\u2006'+unit,
    value: v,
    unit: unit,
   };
  }


  function dist(x,y){
   return Math.sqrt(x*x+y*y);
  }


/*
  U.rotate(pt,t){
   return {
    x: pt.x*Math.cos(t) - pt.y*Math.sin(t);
    y: pt.x*Math.sin(t) + pt.y*Math.cos(t);
   };
  }
*/

  function merge(/*...*/){
   let v = {};
   for( let i = 0; i < arguments.length; i++ ) {
    Object.assign(v,arguments[i]);
   }
   return v;
  }


  {
   let svg = document.createElementNS("http://www.w3.org/2000/svg","svg");
   let svgpt = svg.createSVGPoint();

   U.getScreenCTM = function(el){
    if ( !el ) {
     return svg.createSVGMatrix();
    } else {
     let ctm = U.getScreenCTM(el.parentElement);
     let style = el.currentStyle || getComputedStyle(el,null);
     let transform = style.transform || style.WebkitTransform;
     let transformOrigin = style.transformOrigin
      || style.WebkitTransformOrigin;
     let matrix;
     if ( transform ) { matrix = transform.match(/\(([^\)]+)\)/); }
     let origin;
     if (transformOrigin ) { origin = transformOrigin.split(/\s+/); }
     ctm = ctm.translate(el.offsetLeft,el.offsetTop);
//console.log('offsetTop',el.tagName,el.offsetTop,el.scrollTop);
// scrollしたときダメだ。
     if ( matrix ) {
      let ox = parseFloat(origin[0])
      let oy = parseFloat(origin[1]);
      ctm = ctm.translate(ox,oy);
      let m = svg.createSVGMatrix();
      let a = matrix[1].split(/\s*,\s*/);
      m.a = parseFloat(a[0]); m.b = parseFloat(a[1]);
      m.c = parseFloat(a[2]); m.d = parseFloat(a[3]);
      m.e = parseFloat(a[4]); m.f = parseFloat(a[5]);
      ctm = ctm.multiply(m);
      ctm = ctm.translate(-ox,-oy);
     }
     return ctm;
    }
   };

   U.convertFromScreenCoordinate = function(el,pt){
    let ctm = U.getScreenCTM(el);
    svgpt.x = pt.x;
    svgpt.y = pt.y;
    return svgpt.matrixTransform(ctm.inverse());
   }
  }


  const vendorPrefixNeeded = {
   transformOrigin: 1,
   transform      : 1,
   filter         : 1,
  };
  
  function setStyle(dst,src){
   if ( !src ) { return; }
   for( let property in src ) {
    if ( vendorPrefixNeeded[property] ) {
     let pascal = property.replace(/^([a-z])/,function(match){
      return match.toUpperCase();
     });
     dst['Ms'+pascal] = src[property];
     dst['Moz'+pascal] = src[property];
     dst['Webkit'+pascal] = src[property];
    }
    dst[property] = src[property];
   }
  }


  // measureText
// lineHeight 1?
  {
   let measureBox = document.createElement('div');
   setStyle(measureBox.style,{
    visibility: 'hidden', position:'absolute',
   });
   document.body.appendChild(measureBox);

   U.measureText = function(text,style){
    measureBox.innerHTML =
     `<span>${text}</span><span>a</span>`;
    let box1 = measureBox.firstChild;
    let box2 = box1.nextSibling;
    setStyle(box1.style,{
     border:'solid 1px blue',
     whiteSpace:'nowrap',
    });
    setStyle(box1.style,style);
    setStyle(box2.style,{
     fontFamily:'sans-serif', fontSize:'6px',
    });
    let metrics = {};
    metrics.width = box1.offsetWidth;
    metrics.height = box1.offsetHeight;
    metrics.baselineOffset = box1.offsetHeight - 
     (box2.offsetTop - box1.offsetTop + (box2.offsetHeight*0.8));
    return metrics;
   };
  }


 // misiyou
  // tekitou...
  function styleToString(style){
   let str = '';
   for( let prop in style ) {
    //
    let kebab = prop.replace(/([A-Z])/g,function(match){
    console.log(match);
     return '-'+match.toLowerCase();
    });
    if ( str != '' ) { str += ' '; }
    // 
    let value = style[prop].toString().replace(/"/g,"'");;
    str += `${kebab}: ${value};`;
   }
   return str;
  }


 // scale
  // replace under the container.
  function fitText(container,text){
   if ( text === undefined ) { text = ''; }
   container.innerHTML = `<div style="display:inline-block;
    position:absolute; visibility:hidden; white-space:nowrap;">${text}</div>`;
   let txel = container.firstChild;
   if ( txel.offsetWidth <= 0 || txel.offsetHeight <= 0 ) { return; }
   let scale = Math.min(
    container.offsetWidth/txel.offsetWidth,
    container.offsetHeight/txel.offsetHeight);
   let w = txel.offsetWidth*scale;
   let h = txel.offsetHeight*scale;
   let x = (container.offsetWidth - w)/2;
   let y = (container.offsetHeight- h)/2;
   U.setStyle(txel.style,{
    visibility:'visible',
    left:x+'px', top:y+'px',
    width:txel.offsetWidth+'px', height:txel.offsetHeight+'px',
    transformOrigin:'left top', transform:`scale(${scale})`,
   });
  }


  //
  // "data:image/png;base64,data" -> ["image/png",true,data]
  //
  function splitDataURL(dataURL){
   // data:[<mediatype>][;base64],<data>
   let i = dataURL.indexOf(',');
   if ( i < 0 ) { return; }
   let head = dataURL.substr(0,i);
   let pat = head.match(/^data:([^;]*)(;base64)?$/i);
   if ( !pat ) {
    return;
   }
   return{
    mediaType: pat[1] || "text/plain",
    base64: pat[2] !== undefined,
    data: dataURL.substr(i+1)
   };
  }






  /*
   dataURL
    data:image/png;base64,

   png形式のdataURLをobjectURLにする。
   画面表示のとき、svgの中に背景画像を埋め込む場合
    背景画像にcanvasを使うなら不要か。
   同じ画像をたくさん埋め込む場合。
   未定
  */
/*
未使用
  function dataURLToObjectURL(dataURL){
   let comp = splitDataURL(dataURL);
   if ( !comp ) { return; }
   let data = comp.data;
   if ( comp.base64 ) {
    let str = window.atob(data);
    let bin = new Uint8Array(str.length);
    for( let i = 0; i < str.length; i++ ) {
     bin[i] = str.charCodeAt(i);
    }
    data = bin;
   } else {
    // tukawanai to omou
    return;
   }
   let blob = new Blob([data], {type:comp.mediaType});
   let URL = window.URL || window.webkitURL;
   let objectURL = URL.createObjectURL(blob);
   return objectURL;
  }
*/


  //
  function svgTextToObjectURL(svgText){
   let blob = new Blob([svgText], {type:"image/svg+xml"});
   let URL = window.URL || window.webkitURL;
   let objectURL;
   if ( URL.createObjectURL ) {
    objectURL = URL.createObjectURL(blob);
   }
   return objectURL;
  }


  function revokeObjectURL(objectURL){
   let URL = window.URL || window.webkigURL;
   if ( URL.revokeObjectURL ) {
    URL.revokeObjectURL(objectURL);
   }
  }


/* 未使用
  U.textToDataURL = function(text,callback){
   let blob = new Blob([text], {type:"text/plain"});
   let reader = new FileReader();
   reader.onloadend = function(){
    if ( reader.error ) {
//     console.log(reader.error);
     callback();
    }
    else {
     let dataURL = reader.result;
//  console.log(dataURL);
     callback(dataURL);
    }
   }
   reader.readAsDataURL(blob);
  };
*/


  // file/blob
  U.readAsDataURLP = (file)=>{
   return new Promise((resolve, reject) => {
    let reader = new FileReader();
    reader.onerror = () => {
     reader.abort();
     reject(reader.error);
    };
    reader.onload = () => {
     resolve(reader.result);
    };
    reader.readAsDataURL(file);
   });
  };


  U.readAsTextP = function(file){
   return new Promise((resolve, reject) => {
    let reader = new FileReader();
    reader.onerror = () => {
     reader.abort();
     reject(reader.error);
    };
    reader.onload = () => {
     resolve(reader.result);
    };
    reader.readAsText(file);
   });
  };


  U.dataURLToImageP = async function(dataURL) {
   if ( !dataURL ) {
    throw('dataURLToImage#1');
   }
   let img = await new Promise((resolve, reject) => {
    let img = new Image;
    img.onerror = () => {
     reject(Error("dataURLToImageP"));
    };
    img.onload = () => {
     resolve(img);
    };
    img.src = dataURL;
   });
   return img;
  };


  U.svgToDataURLP = async(svgText)=>{
   let blob = new Blob([svgText], {type:"image/svg+xml"});
   return await U.readAsDataURLP(blob);
  };


  U.loadImageP = async function(file){
   let image;
   let dataURL = await U.readAsDataURLP(file);
   if ( dataURL ) {
    image = await U.dataURLToImageP(dataURL);
    if ( (image.width === 0 || image.height === 0) ) {
     if ( file.type === 'image/svg+xml' ) {
      // Firefox!!
      // WaterMarkの方は動いたがメインのほうが別の場所で引っかかっているもよう
      let text = await U.readAsTextP(file);
      // うーん
      let m = text.match(
        /viewbox\s*=\s*"\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*"/i);
      if ( m ) {
       image.width = m[3];
       image.height = m[4];
      }
     }
    }
    if ( image.width === 0 || image.height === 0 ) {
     throw('image width');
    }
   }
   return image;
  };









// saveText
  U.downloadText = function(fileName, text, type){
   if ( !type ) { type = 'text/plain'; }
   let blob = new Blob([text], {type:type});
   let URL = window.URL || window.webkitURL;
   let objectURL;
   if ( URL.createObjectURL ) {
    objectURL = URL.createObjectURL(blob);
   } else {
    console.log('createObjectURL');
    return;
   }
   let a = document.createElement('a');
   a.download = fileName;
   a.href = objectURL;
   let e = document.createEvent("MouseEvents");
   e.initMouseEvent("click", true, true, window,
    0, 0, 0, 0, 0, false, false, false,false, 0, null);
   a.dispatchEvent(e);
   URL.revokeObjectURL(objectURL);
  };


  // multiple
  U.openFileP = function(property){
   return new Promise((resolve, reject)=>{
    let input = document.createElement('input');
    input.type = 'file';
    input.style.display = 'none';
    document.body.appendChild(input);
    for( let p in property ) {
     input[p] = property[p];
    }
    function onchange(ev) {
// これ
     input.removeEventListener('change',onchange,false);
     document.body.removeChild(input);
     resolve(ev.target.files);
    }
    input.addEventListener('change',onchange,false);
    let e = document.createEvent("MouseEvents");
    e.initMouseEvent("click", true, true, window,
     0, 0, 0, 0, 0, false, false, false,false, 0, null);
    input.dispatchEvent(e);
   });
  };


  U.sleepMSP = function(ms){
   return new Promise((resolve, reject) => {
    setTimeout(resolve,ms);
   });
  };


  return U;

 });
})();
