/*
 watermark.js

 TODO:
  tile, image
  回転


 クラス化
 メニューを別にしないと難しいか

  values


 layer
  layer1 watermark
  layer2 scalebar
  ...
  
 

*/
(function(){
 'use strict';
 const Ld = sLoader;
 Ld.module('waterMark',function(){

  const Lp   = Ld.use('layerParts');

  const Wm = Lp.derive();
  Wm._className = 'WaterMark';

  const M    = Ld.ref('main');
  const Mp   = Ld.use('mainParam');
  const U    = Ld.use('utility');
  const Mn   = Ld.use('menu');
  const Cp   = Ld.use('cropParam');

  Wm.sig = 'WaterMark';


  const filterNames = [
   'none',
   'softlight',
   'shadow',
   'emboss',
  ];

  let filterImpl = [];

  for( let i = 0; i < filterNames.length; i++ ) {
   filterImpl[filterNames[i]] = Ld.use( 'filter-'+filterNames[i] );
  }


  function makeSelText(id,text,filterName){
   let filter = filterImpl[filterName];
   let backId = `${id}_${filterName}_back`;
   let svg = `<svg viewBox="${0} ${0} ${560} ${180}" width="560" height="180"
     opacity="0.8" >
    <defs>
     <linearGradient id="${backId}"
       x1="0%" x2="0%" y1="100%" y2="0%">
      <stop offset="5%" stop-color="#CCF" />
      <stop offset="95%" stop-color="#89F" />
     </linearGradient>`;
   let filterId = `${id}_${filterName}_sel`;
   let vb = {
    x:0, y:0, width:560, height:180,
   };
   let filterSvg = filter.makeSvgText( filterId, vb, vb );
   let filterAttr = filterSvg != '' ? `filter="url(#${filterId})"`: '';
   svg += filterSvg;
   svg +=
    `</defs>
    <g style="isolation:isolate">
     <rect x="0" y="0" width="100%" height="100%" fill="url(#${backId})" />
     <g ${filterAttr} style="${filter.blendMode}" >
      <text font-size="${120}" font-weight="600" fill="white" x="16" y="144" >
       ${U.htmlEscape(text)}
      </text>
     </g>
    </g>
   </svg>`;
   return svg;
  };


  Wm.defineMenu = function(){
   Wm._super.defineMenu.call(this);
   let obj = this;
   this.addMenu({
    show          : Mn.visibilitySwitch({priority:90}),
    transform       :
      Mn.CTransform.create({priority:85,
       buttonText: 'Move/Resize',
       zoom:{ min:10, max:100, default:40 },
       ontransformstart:function(){ obj.ontransformstart(); },
       ontransformend:function(){ obj.ontransformend(); },
       onopen: function() { obj.ontransformopen(); },
       onclose: function() { obj.ontransformclose(); },
       screenToInternalCoordinate:function(pos){
        let p = U.convertFromScreenCoordinate(M.paneEl,pos);
        let cw = M.scale*Cp.width;
        let ch = M.scale*Cp.height;
        return { x:(p.x-cw/2)/M.scale, y:(p.y-ch/2)/M.scale };
       },
      }),
    opacity       : Mn.opacity({priority:80}),
    color         : Mn.bwColor({priority:70, default:1}),
    effect        :
      Mn.CSelect.create({ priority:15,
       default:'none',
       buttonText:'Effect',
       cellHeight:180,
       cellPadH:0,
       cellPadV:0,
       // width
       list: filterNames.map(function(name){
        return {
         value:name,
         htmlText:makeSelText(obj.id,name,name)
        };
       }),
      }),
   });
  };


// transform
  Wm.ontransformopen = function(){
   this.highlightStatus = true;
   this.dirty = 1;
   M.disp();
  };

  Wm.ontransformclose = function(){
   this.highlightStatus = false;
   this.dirty = 1;
   M.disp();
  };

  Wm.ontransformstart = function(){
   let rect = this.getGeometry();
   if ( rect ) {
    this.values.transform.x = rect.centerX;
    this.values.transform.y = rect.centerY;
    this.dirty = 1;
    M.disp();
   }
  };

  Wm.ontransformend = function(){
  };

  Wm.makeSvgText = function(final,p){
   let V = this.values;
   let svg = "";
   let srcbb = this.getSourceBoundingBox();
   if ( srcbb ) {
    let scale = p.cRect.diagonal;
//scale=1000;

    let filter = filterImpl[
     !final && Mp.view.scale >= 2? 'none': V.effect
    ];
    let ex = 2;
    let srcbbPx = {
     x: scale*srcbb.x-ex, y: scale*srcbb.y-ex,
     width:scale*srcbb.width+2*ex, height:scale*srcbb.height+2*ex,
    };
    let viewBox = filter.getViewBox(srcbbPx);
/*
    `<svg viewBox="${viewBox.x} ${viewBox.y} ${viewBox.width} ${viewBox.height}"
      x="${viewBox.x+p.cRect.width/2}" y="${viewBox.y+p.cRect.height/2}"
      width="${viewBox.width}" height="${viewBox.height}"
      style="position:absolute; left:${viewBox.x+p.cRect.width/2}px;
        top:${viewBox.y+p.cRect.height/2}px; ${filter.blendMode}"
*/
    
    let cw = scale*Cp.width;
    let ch = scale*Cp.height;
    svg =
    `<svg viewBox="${-cw/2} ${-ch/2} ${cw} ${ch}"
      x="0" y="0" width="100%" height="100%"
      style="position:absolute; left:0;
        top:0; ${filter.blendMode}"
      opacity="${V.opacity/100}" >
      xmlns="http://www.w3.org/2000/svg"
      xmlns:xlink="http://www.w3.org/1999/xlink"
     <defs>`;
    let invFilter = '';
    if ( !V.color || V.makeAlpha ) {
     let invFilterId = this.id+'_inv';
     invFilter = `filter="url(#${invFilterId})"`;
     svg += `<filter id="${invFilterId}"
      filterUnits="userSpaceOnUse" x="${viewBox.x}" y="${viewBox.y}"
        width="${viewBox.width}" height="${viewBox.height}" >`;
     if ( V.makeAlpha ) {
      svg += `<feColorMatrix values="1 0 0 0 0 0 1 0 0 0 0 0 1 0 0
           0.213 0.787 0.072 0 0" />`;
     }
     if ( !V.color ) {
      svg += `<feColorMatrix
        values="-1 0 0 0 1 0 -1 0 0 1 0 0 -1 0 1 0 0 0 1 0"
        color-interpolation-filters="sRGB"
        />`
     }
     svg += '</filter>';
    }
    let effectFilterId = this.id+'_effect';
    let effectSvg = filter.makeSvgText( effectFilterId, viewBox, srcbbPx );
    let effectFilter = 
     effectSvg != ''?  `filter="url(#${effectFilterId})"`: '';
    svg += effectSvg;
    svg += '</defs>';
    svg +=
     `<g ${effectFilter}>
       <g ${invFilter}>
        ${this.makeSvgTextContent(final,scale)}
       </g>
      </g>
     </svg>`;
   }
   return svg;
  };


/*
  Wm.highlight = function(){
   let box = this.getHighlightBox();
   if ( this.highlightStatus && box ) {
    let frame = this.highlightFrame;
    if ( !frame ) {
     frame = this.highlightFrame = document.createElement('div');
     U.setStyle(frame.style,{
      position:'absolute', boxSizing:'border-box', zIndex:9,
      borderStyle:'solid', borderColor:'blue', opacity:'0.4',
     });
     M.paneEl.appendChild(frame);
    }
    let scale = M.cRect.diagonal;
    U.setStyle(frame.style,{
     left: (scale*box.x + M.cRect.width/2)+'px',
     top : (scale*box.y + M.cRect.height/2)+'px',
     width: (scale*box.width)+'px',
     height: (scale*box.height)+'px',
     borderWidth:(scale*4/1000)+'px',
    });
   } else {
    if ( this.highlightFrame ) {
     M.paneEl.removeChild(this.highlightFrame);
     this.highlightFrame = undefined;
    }
   }
  };
*/


  return Wm;

 });
})();
